@@ -3,7 +3,8 @@ import logging
 from datetime import datetime
 from operator import itemgetter
 
-from pyelasticsearch import ElasticSearch
+from elasticsearch import Elasticsearch
+from elasticsearch.helpers import bulk_index
 
 from elasticutils._version import __version__  # noqa
 
@@ -12,12 +13,23 @@ log = logging.getLogger('elasticutils')
 
 
 # Note: Don't change these--they're not part of the API.
-DEFAULT_URLS = ['http://localhost:9200']
+DEFAULT_URLS = ['localhost']
 DEFAULT_DOCTYPES = None
 DEFAULT_INDEXES = None
 DEFAULT_TIMEOUT = 5
 
 
+#: Valid facet types
+FACET_TYPES = [
+    'date_histogram',
+    'filter',
+    'histogram',
+    'query',
+    'range',
+    'statistical',
+    'terms',
+]
+
 #: Maps ElasticUtils field actions to their Elasticsearch query names.
 QUERY_ACTION_MAP = {
     None: 'term',  # Default to term
@@ -69,11 +81,11 @@ def _build_key(urls, timeout, **settings):
     # Order the settings by key and then turn it into a string with
     # repr. There are a lot of edge cases here, but the worst that
     # happens is that the key is different and so you get a new
-    # ElasticSearch. We'll probably have to tweak this.
+    # Elasticsearch. We'll probably have to tweak this.
     settings = sorted(settings.items(), key=lambda item: item[0])
     settings = repr([(k, v) for k, v in settings])
 
-    # pyelasticsearch allows urls to be a string, so we make sure to
+    # elasticsearch allows urls to be a string, so we make sure to
     # account for that when converting whatever it is into a tuple.
     if isinstance(urls, basestring):
         urls = (urls,)
@@ -90,41 +102,40 @@ _cached_elasticsearch = {}
 
 
 def get_es(urls=None, timeout=DEFAULT_TIMEOUT, force_new=False, **settings):
-    """Create a pyelasticsearch `ElasticSearch` object and return it.
+    """Create an elasticsearch `Elasticsearch` object and return it.
 
-    This will aggressively re-use `ElasticSearch` objects with the
+    This will aggressively re-use `Elasticsearch` objects with the
     following rules:
 
     1. if you pass the same argument values to `get_es()`, then it
-       will return the same `ElasticSearch` object
+       will return the same `Elasticsearch` object
     2. if you pass different argument values to `get_es()`, then it
-       will return different `ElasticSearch` object
-    3. it caches each `ElasticSearch` object that gets created
+       will return different `Elasticsearch` object
+    3. it caches each `Elasticsearch` object that gets created
     4. if you pass in `force_new=True`, then you are guaranteed to get
-       a fresh `ElasticSearch` object AND that object will not be
+       a fresh `Elasticsearch` object AND that object will not be
        cached
 
     :arg urls: list of uris; Elasticsearch hosts to connect to,
         defaults to ``['http://localhost:9200']``
     :arg timeout: int; the timeout in seconds, defaults to 5
-    :arg force_new: Forces get_es() to generate a new ElasticSearch
+    :arg force_new: Forces get_es() to generate a new Elasticsearch
         object rather than pulling it from cache.
-    :arg settings: other settings to pass into ElasticSearch
+    :arg settings: other settings to pass into Elasticsearch
         constructor; See
-        `<http://pyelasticsearch.readthedocs.org/en/latest/api/>`_ for
-        more details.
+        `<http://elasticsearch-py.readthedocs.org/>`_ for more details.
 
     Examples::
 
-        # Returns cached ElasticSearch object
+        # Returns cached Elasticsearch object
         es = get_es()
 
-        # Returns a new ElasticSearch object
+        # Returns a new Elasticsearch object
         es = get_es(force_new=True)
 
-        es = get_es(urls=['http://localhost:9200'])
+        es = get_es(urls=['localhost'])
 
-        es = get_es(urls=['http://localhost:9200'], timeout=10,
+        es = get_es(urls=['localhost:9200'], timeout=10,
                     max_retries=3)
 
     """
@@ -140,7 +151,7 @@ def get_es(urls=None, timeout=DEFAULT_TIMEOUT, force_new=False, **settings):
         if key in _cached_elasticsearch:
             return _cached_elasticsearch[key]
 
-    es = ElasticSearch(urls, timeout=timeout, **settings)
+    es = Elasticsearch(urls, timeout=timeout, **settings)
 
     if not force_new:
         # We don't need to rebuild the key here since we built it in
@@ -173,6 +184,8 @@ def _process_facets(facets, flags):
         facet_type = {'terms': {'field': fieldname}}
         if flags.get('global_'):
             facet_type['global'] = flags['global_']
+        if flags.get('size'):
+            facet_type['terms']['size'] = flags['size']
         elif flags.get('filtered'):
             # Note: This is an indicator that the facet_filter should
             # get filled in later when we know all the filters.
@@ -191,24 +204,41 @@ def _facet_counts(items):
 
     """
     facets = {}
-    for key, val in items:
-        if val['_type'] == 'terms':
-            facets[key] = [v for v in val['terms']]
-        elif val['_type'] == 'range':
-            facets[key] = [v for v in val['ranges']]
-        elif val['_type'] == 'histogram':
-            facets[key] = [v for v in val['entries']]
-        elif val['_type'] == 'date_histogram':
-            facets[key] = [v for v in val['entries']]
-        elif val['_type'] in ('filter', 'query', 'statistical'):
-            facets[key] = val
-        else:
-            raise InvalidFacetType(
-                'Facet _type "%s". key "%s" val "%r"' %
-                (val['_type'], key, val))
+    for name, data in items:
+        facets[name] = FacetResult(name, data)
     return facets
 
 
+class FacetResult(object):
+    def __init__(self, name, data, *args, **kwargs):
+        if data['_type'] not in FACET_TYPES:
+            raise InvalidFacetType(
+                'Facet _type "{0}". key "{1}" val "{2}"'.format(
+                    data['_type'], name, data))
+
+        self._data = data
+        self.__dict__.update(data)
+
+        for attr in ('entries', 'ranges', 'terms'):
+            if attr in data:
+                self.data = getattr(self, attr)[:]
+                break
+        else:
+            self.data = []
+
+    def __repr__(self):
+        return repr(self._data)
+
+    def __iter__(self):
+        return iter(self.data)
+
+    def __getitem__(self, key):
+        try:
+            return getattr(self, key)
+        except AttributeError as exc:
+            raise KeyError(exc.message)
+
+
 class F(object):
     """
     Filter objects.
@@ -387,15 +417,21 @@ class PythonMixin(object):
 
         """
         if isinstance(obj, basestring):
-            if len(obj) == 19:
-                # Note: ES seems to support other date/datetime
-                # formats, but I wasn't able to create a situation
-                # where it returns something in a different shape in
-                # the results.
+            if len(obj) == 26:
+                try:
+                    return datetime.strptime(obj, '%Y-%m-%dT%H:%M:%S.%f')
+                except (TypeError, ValueError):
+                    pass
+            elif len(obj) == 19:
                 try:
                     return datetime.strptime(obj, '%Y-%m-%dT%H:%M:%S')
                 except (TypeError, ValueError):
                     pass
+            elif len(obj) == 10:
+                try:
+                    return datetime.strptime(obj, '%Y-%m-%d')
+                except (TypeError, ValueError):
+                    pass
 
         elif isinstance(obj, dict):
             for key, val in obj.items():
@@ -412,20 +448,27 @@ class S(PythonMixin):
 
     The API for `S` takes inspiration from Django's QuerySet.
 
-    `S` can be either typed or untyped. An untyped `S` returns dict
-    results by default.
+    `S` can be either typed or untyped. An untyped `S` returns results
+    as an iterable of :py:class:`elasticutils.DefaultMappingType`
+    instances.
 
     An `S` is lazy in the sense that it doesn't do an Elasticsearch
-    search request until it's forced to evaluate by either iterating
-    over it, calling ``.count``, doing ``len(s)``, or calling
-    ``.facet_count``.
+    search request until it's forced to evaluate by:
+
+    1. use the :py:class:`elasticutils.S` in an iterable context
+    2. call :py:func:`len` on a :py:class:`elasticutils.S`
+    3. call the :py:meth:`elasticutils.S.execute`,
+       :py:meth:`elasticutils.S.everything`,
+       :py:meth:`elasticutils.S.count`,
+       :py:meth:`elasticutils.S.suggestions` or
+       :py:meth:`elasticutils.S.facet_counts` methods
 
 
     **Adding support for other queries**
 
     You can add support for queries that S doesn't have support for by
-    subclassing S with a method called ``process_query_ACTION``.  This
-    method takes a key, value and an action.
+    subclassing S with a method called ``process_query_<ACTION>``.
+    This method takes a key, value and an action.
 
     For example::
 
@@ -464,7 +507,7 @@ class S(PythonMixin):
     **Adding support for other filters**
 
     You can add support for filters that S doesn't have support for by
-    subclassing S with a method called ``process_filter_ACTION``.
+    subclassing S with a method called ``process_filter_<ACTION>``.
     This method takes a key, value and an action.
 
     For example::
@@ -483,7 +526,7 @@ class S(PythonMixin):
     def __init__(self, type_=None):
         """Create and return an S.
 
-        :arg type_: class; the model that this S is based on
+        :arg type_: class; the MappingType for this S
 
         """
         self.type = type_
@@ -496,11 +539,11 @@ class S(PythonMixin):
 
     def __repr__(self):
         try:
-            return '<S {0}>'.format(repr(self._build_query()))
+            return '<S {0}>'.format(repr(self.build_search()))
         except RuntimeError:
-            # This happens when you're debugging _build_query and try
-            # to repr the instance you're calling it on. Then that
-            # calls _build_query and ...
+            # This can happen when you're debugging build_search() and
+            # try to repr the instance you're calling it on. Then that
+            # calls build_search() and CLOWN SHOES!
             return repr(self.steps)
 
     def _clone(self, next_step=None):
@@ -514,13 +557,13 @@ class S(PythonMixin):
         return new
 
     def es(self, **settings):
-        """Return a new S with specified ElasticSearch settings.
+        """Return a new S with specified Elasticsearch settings.
 
-        This allows you to configure the ElasticSearch object that gets
+        This allows you to configure the Elasticsearch object that gets
         used to execute the search.
 
         :arg settings: the settings you'd use to build the
-            ElasticSearch---same as what you'd pass to
+            Elasticsearch---same as what you'd pass to
             :py:func:`get_es`.
 
         """
@@ -868,6 +911,13 @@ class S(PythonMixin):
         """
         Return a new S instance with facet args combined with existing
         set.
+
+        :arg args: The list of facets to return.
+
+        Additional keyword options:
+
+        * ``size`` -- Maximum number of terms to return for each facet.
+
         """
         return self._clone(next_step=('facet', (args, kw)))
 
@@ -889,8 +939,8 @@ class S(PythonMixin):
         * ``pre_tags`` -- List of tags before highlighted portion
         * ``post_tags`` -- List of tags after highlighted portion
 
-        Results will have a ``_highlight`` property which contains
-        the highlighted field excerpts.
+        Results will have a ``highlight`` attribute on the ``es_meta``
+        object which contains the highlighted field excerpts.
 
         For example::
 
@@ -898,8 +948,8 @@ class S(PythonMixin):
                     .highlight('title', 'content'))
 
             for result in q:
-                print result._highlight['title']
-                print result._highlight['content']
+                print result.es_meta.highlight['title']
+                print result.es_meta.highlight['content']
 
 
         If you pass in ``None``, it will clear the highlight.
@@ -925,6 +975,46 @@ class S(PythonMixin):
         """
         return self._clone(next_step=('highlight', (fields, kwargs)))
 
+    def search_type(self, search_type):
+        """Set Elasticsearch search type for distributed search behaviour.
+
+        :arg search_type: The search type to set.
+
+        The search type affects how much results are fetched from each shard,
+        and how are they then merged back. This can affect the accuracy of the
+        results and the execution speed.
+
+        For the list of possible values and additional documentation,
+        consult the Elasticsearch reference:
+        http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/search-request-search-type.html
+
+        If called multiple times, the last search type will be in effect.
+
+        """
+        return self._clone(next_step=('search_type', search_type))
+
+    def suggest(self, name, term, **kwargs):
+        """Set suggestion options.
+
+        :arg name: The name to use for the suggestions.
+        :arg term: The term to suggest similar looking terms for.
+
+        Additional keyword options:
+
+        * ``field`` -- The field to base suggestions upon, defaults to _all
+
+        Results will have a ``_suggestions`` property containing the
+        suggestions for all terms.
+
+        .. Note::
+
+           Suggestions are only supported since Elasticsearch 0.90.
+
+           Calling this multiple times will add multiple suggest clauses to
+           the query.
+        """
+        return self._clone(next_step=('suggest', (name, term, kwargs)))
+
     def extra(self, **kw):
         """
         Return a new S instance with extra args combined with existing
@@ -942,8 +1032,8 @@ class S(PythonMixin):
         return new
 
     def __getitem__(self, k):
+        """Handles slice and indexes for Elasticsearch results"""
         new = self._clone()
-        # TODO: validate numbers and ranges
         if isinstance(k, slice):
             new.start, new.stop = k.start or 0, k.stop
             return new
@@ -951,10 +1041,17 @@ class S(PythonMixin):
             new.start, new.stop = k, k + 1
             return list(new)[0]
 
-    def _build_query(self):
-        """
-        Loop self.steps to build the query format that will be sent to
-        Elasticsearch, and return it as a dict.
+    def build_search(self):
+        """Builds the Elasticsearch search body represented by this S.
+
+        Loop over self.steps to build the search body that will be
+        sent to Elasticsearch. This returns a Python dict.
+
+        If you want the JSON that actually gets sent, then pass the return
+        value through :py:func:`elasticutils.utils.to_json`.
+
+        :returns: a Python dict
+
         """
         filters = []
         filters_raw = None
@@ -968,8 +1065,11 @@ class S(PythonMixin):
         demote = None
         highlight_fields = set()
         highlight_options = {}
+        suggestions = {}
         explain = False
         as_list = as_dict = False
+        search_type = None
+
         for action, value in self.steps:
             if action == 'order_by':
                 sort = []
@@ -1014,6 +1114,10 @@ class S(PythonMixin):
                 else:
                     highlight_fields |= set(value[0])
                 highlight_options.update(value[1])
+            elif action == 'search_type':
+                search_type = value
+            elif action == 'suggest':
+                suggestions[value[0]] = (value[1], value[2])
             elif action in ('es', 'indexes', 'doctypes', 'boost'):
                 # Ignore these--we use these elsewhere, but want to
                 # make sure lack of handling it here doesn't throw an
@@ -1067,7 +1171,7 @@ class S(PythonMixin):
             # None as a shell, so if it's explicitly set to None, then
             # we update it.
             for facet in facets.values():
-                if facet.get('facet_filter', 1) is None:
+                if facet.get('facet_filter', 1) is None and 'filter' in qs:
                     facet['facet_filter'] = qs['filter']
 
         if facets_raw:
@@ -1087,7 +1191,16 @@ class S(PythonMixin):
         if explain:
             qs['explain'] = True
 
+        for suggestion, (term, kwargs) in suggestions.iteritems():
+            qs.setdefault('suggest', {})[suggestion] = {
+                'text': term,
+                'term': {
+                    'field': kwargs.get('field', '_all'),
+                },
+            }
+
         self.fields, self.as_list, self.as_dict = fields, as_list, as_dict
+        self.search_type = search_type
         return qs
 
     def _build_highlight(self, fields, options):
@@ -1169,7 +1282,7 @@ class S(PythonMixin):
         return rv
 
     def _process_query(self, query):
-        """Takes a key/val pair and returns ES JSON API"""
+        """Takes a key/val pair and returns the Elasticsearch code for it"""
         key, val = query
         field_name, field_action = split_field_action(key)
 
@@ -1288,16 +1401,16 @@ class S(PythonMixin):
         return self._results_cache
 
     def get_es(self, default_builder=get_es):
-        """Returns the ElasticSearch object to use.
+        """Returns the Elasticsearch object to use.
 
         :arg default_builder: The function that takes a bunch of
-            arguments and generates a pyelasticsearch ElasticSearch
+            arguments and generates a elasticsearch Elasticsearch
             object.
 
         .. Note::
 
            If you desire special behavior regarding building the
-           ElasticSearch object for this S, subclass S and override
+           Elasticsearch object for this S, subclass S and override
            this method.
 
         """
@@ -1308,7 +1421,7 @@ class S(PythonMixin):
             if action == 'es':
                 args.update(**value)
 
-        # TODO: store the ElasticSearch on the S if we've already
+        # TODO: store the Elasticsearch on the S if we've already
         # created one since we don't need to do it multiple times.
         return default_builder(**args)
 
@@ -1342,7 +1455,7 @@ class S(PythonMixin):
         Build query and passes to Elasticsearch, then returns the raw
         format returned.
         """
-        qs = self._build_query()
+        qs = self.build_search()
         es = self.get_es()
 
         index = self.get_indexes()
@@ -1352,75 +1465,84 @@ class S(PythonMixin):
             raise BadSearch(
                 'You must specify an index if you are specifying doctypes.')
 
-        hits = es.search(qs,
+        extra_search_kwargs = {}
+        if self.search_type:
+            extra_search_kwargs['search_type'] = self.search_type
+
+        hits = es.search(body=qs,
                          index=self.get_indexes(),
-                         doc_type=self.get_doctypes())
+                         doc_type=self.get_doctypes(),
+                         **extra_search_kwargs)
 
         log.debug('[%s] %s' % (hits['took'], qs))
         return hits
 
     def count(self):
         """
-        Executes search and returns number of results as an integer.
+        Returns the total number of results Elasticsearch thinks will
+        match this search.
 
         :returns: integer
 
         For example:
 
-        >>> s = S().query(name__prefix='Jimmy')
-        >>> count = s.count()
+        >>> all_jimmies = S().query(name__prefix='Jimmy').count()
 
         """
-        if self._results_cache:
+        if self._results_cache is not None:
             return self._results_cache.count
         else:
             return self[:0].raw()['hits']['total']
 
     def __len__(self):
-        """
-        Executes search and returns the number of results you'd get.
+        """Executes search and returns the number of results you'd get.
 
-        Executes search and returns number of results as an integer.
+        Executes search and returns number of results returned as an
+        integer.
 
         :returns: integer
 
         For example:
 
-        >>> s = S().query(name__prefix='Jimmy')
-        >>> count = len(s)
-        >>> results = s().execute()
-        >>> count = len(results)
-        True
+        >>> some_s = S().query(name__prefix='Jimmy')
+        >>> length = len(some_s)
 
-        .. Note::
-
-           This is very different than calling ``.count()``. If you
-           call ``.count()`` you get the total number of results
-           that Elasticsearch thinks matches your search. If you call
-           ``len(s)``, then you get the number of results you'd get
-           if you executed the search. This factors in slices and
-           default from and size values.
+        This is very different than calling
+        :py:meth:`elasticutils.S.count`. If you call
+        :py:meth:`elasticutils.S.count` you get the total number of
+        results that Elasticsearch thinks matches your search. If you
+        call ``len()``, then you get the number of results you got
+        from the search which factors in slices and default from and
+        size values.
 
         """
         return len(self._do_search())
 
     def all(self):
+        """No-op that returns a clone of self
+
+        This is here to make it more Django QuerySet-like and work
+        with better with things that accept QuerySets.
+
         """
-        Executes search and returns ALL search results.
+        return self._clone()
+
+    def everything(self):
+        """Executes search and returns ALL search results.
 
         :returns: `SearchResults` instance
 
         For example:
 
         >>> s = S().query(name__prefix='Jimmy')
-        >>> all_results = s.all()
+        >>> all_results = s.everything()
 
         .. Warning::
 
-           This returns ALL search results. The way it does this is by
-           calling ``.count()`` first to figure out how many to return,
-           then by slicing by that size and returning a list of ALL
-           search results.
+           This returns ALL possible search results. The way it does
+           this is by calling ``.count()`` first to figure out how
+           many to return, then by slicing by that size and returning
+           ALL possible search results.
 
            Don't use this if you've got 1000s of results!
 
@@ -1472,6 +1594,20 @@ class S(PythonMixin):
         """
         return _facet_counts(self._raw_facets().items())
 
+    def suggestions(self):
+        """
+        Executes search and returns suggestions.
+
+        >>> s = S().query(name='Aice').suggest(name='Aice')
+        >>> suggestions = s.suggestions()['name']
+
+        .. Note::
+
+           Suggestions are only supported since Elasticsearch 0.90.
+
+        """
+        return self._do_search().response.get('suggest', {})
+
 
 class MLT(PythonMixin):
     """Represents a lazy Elasticsearch More Like This API request.
@@ -1500,7 +1636,7 @@ class MLT(PythonMixin):
             listed in s.get_indexes().
         :arg doctype: The doctype to use. Falls back to the first
             doctype listed in s.get_doctypes().
-        :arg es: The `ElasticSearch` object to use. If you don't
+        :arg es: The `Elasticsearch` object to use. If you don't
             provide one, then it will create one for you.
         :arg query_params: Any additional query parameters for the
             more like this call.
@@ -1547,12 +1683,12 @@ class MLT(PythonMixin):
         return len(self._do_search())
 
     def get_es(self):
-        """Returns an `ElasticSearch`.
+        """Returns an `Elasticsearch`.
 
-        * If there's an s, then it returns that `ElasticSearch`.
+        * If there's an s, then it returns that `Elasticsearch`.
         * If the es was provided in the constructor, then it returns
-          that `ElasticSearch`.
-        * Otherwise, it creates a new `ElasticSearch` and returns
+          that `Elasticsearch`.
+        * Otherwise, it creates a new `Elasticsearch` and returns
           that.
 
         Override this if that behavior isn't correct for you.
@@ -1565,7 +1701,7 @@ class MLT(PythonMixin):
 
     def raw(self):
         """
-        Build query and passes to `ElasticSearch`, then returns the raw
+        Build query and passes to `Elasticsearch`, then returns the raw
         format returned.
         """
         es = self.get_es()
@@ -1573,10 +1709,11 @@ class MLT(PythonMixin):
         params = dict(self.query_params)
         mlt_fields = self.mlt_fields or params.pop('mlt_fields', [])
 
-        body = self.s._build_query() if self.s else ''
+        body = self.s.build_search() if self.s else ''
 
-        hits = es.more_like_this(
-            self.index, self.doctype, self.id, mlt_fields, body, **params)
+        hits = es.mlt(
+            index=self.index, doc_type=self.doctype, id=self.id,
+            mlt_fields=mlt_fields, body=body, **params)
 
         log.debug(hits)
 
@@ -1711,20 +1848,31 @@ class ObjectSearchResults(SearchResults):
         return self.objects.__iter__()
 
 
+class Metadata(object):
+    def __init__(self, **kwargs):
+        self.__dict__.update(kwargs)
+
+
 def decorate_with_metadata(obj, result):
-    """Return obj decorated with result-scope metadata."""
-    # Elasticsearch id
+    """Return obj decorated with es_meta object"""
+    # Create es_meta object with Elasticsearch metadata about this
+    # search result
+    obj.es_meta = Metadata(
+        # Elasticsearch id
+        id=result.get('_id', 0),
+        # Source data
+        source=result.get('_source', {}),
+        # The search result score
+        score=result.get('_score', None),
+        # The document type
+        type=result.get('_type', None),
+        # Explanation of score
+        explanation=result.get('_explanation', {}),
+        # Highlight bits
+        highlight=result.get('highlight', {})
+    )
+    # Put the id on the object for convenience
     obj._id = result.get('_id', 0)
-    # Source data
-    obj._source = result.get('_source', {})
-    # The search result score
-    obj._score = result.get('_score')
-    # The document type
-    obj._type = result.get('_type')
-    # Explanation structure
-    obj._explanation = result.get('_explanation', {})
-    # Highlight bits
-    obj._highlight = result.get('highlight', {})
     return obj
 
 
@@ -1737,10 +1885,10 @@ class MappingType(object):
 
     To extend this class:
 
-    1. implement ``get_index``.
-    2. implement ``get_mapping_type_name``.
-    3. if this ties back to a model, implement ``get_model`` and
-       possibly also ``get_object``.
+    1. implement ``get_index()``.
+    2. implement ``get_mapping_type_name()``.
+    3. if this ties back to a model, implement ``get_model()`` and
+       possibly also ``get_object()``.
 
     For example::
 
@@ -1831,7 +1979,11 @@ class MappingType(object):
         """Return the model class related to this MappingType.
 
         This can be any class that has an instance related to this
-        Mappingtype by id.
+        MappingType by id.
+
+        For example, if you're using Django and your MappingType is
+        related to a Django model--this should return the Django
+        model.
 
         By default, raises NoModelError.
 
@@ -1895,11 +2047,11 @@ class Indexable(object):
 
     @classmethod
     def get_es(cls):
-        """Returns an ElasticSearch object
+        """Returns an Elasticsearch object
 
         Override this if you need special functionality.
 
-        :returns: a pyelasticsearch `ElasticSearch` instance
+        :returns: a elasticsearch `Elasticsearch` instance
 
         """
         return get_es()
@@ -1982,8 +2134,8 @@ class Indexable(object):
         :arg overwrite_existing: if ``True`` overwrites existing documents
              of the same ID and doctype
 
-        :arg es: The `ElasticSearch` to use. If you don't specify an
-            `ElasticSearch`, it'll use `cls.get_es()`.
+        :arg es: The `Elasticsearch` to use. If you don't specify an
+            `Elasticsearch`, it'll use `cls.get_es()`.
 
         :arg index: The name of the index to use. If you don't specify one
             it'll use `cls.get_index()`.
@@ -2001,12 +2153,11 @@ class Indexable(object):
         if index is None:
             index = cls.get_index()
 
-        es.index(
-            index,
-            cls.get_mapping_type_name(),
-            document,
-            id=id_,
-            overwrite_existing=overwrite_existing)
+        kw = {}
+        if not overwrite_existing:
+            kw['op_type'] = 'create'
+        es.index(index=index, doc_type=cls.get_mapping_type_name(),
+                 body=document, id=id_, **kw)
 
     @classmethod
     def bulk_index(cls, documents, id_field='id', es=None, index=None):
@@ -2022,8 +2173,8 @@ class Indexable(object):
         :arg id_field: The name of the field to use as the document
             id. This defaults to 'id'.
 
-        :arg es: The `ElasticSearch` to use. If you don't specify an
-            `ElasticSearch`, it'll use `cls.get_es()`.
+        :arg es: The `Elasticsearch` to use. If you don't specify an
+            `Elasticsearch`, it'll use `cls.get_es()`.
 
         :arg index: The name of the index to use. If you don't specify one
             it'll use `cls.get_index()`.
@@ -2041,10 +2192,15 @@ class Indexable(object):
         if index is None:
             index = cls.get_index()
 
-        es.bulk_index(index,
-                      cls.get_mapping_type_name(),
-                      documents,
-                      id_field)
+        documents = (dict(d, _id=d[id_field]) for d in documents)
+
+        bulk_index(
+            es,
+            documents,
+            index=index,
+            doc_type=cls.get_mapping_type_name(),
+            raise_on_error=True
+        )
 
     @classmethod
     def unindex(cls, id_, es=None, index=None):
@@ -2053,8 +2209,8 @@ class Indexable(object):
         :arg id_: The Elasticsearch id for the document to remove from
             the index.
 
-        :arg es: The `ElasticSearch` to use. If you don't specify an
-            `ElasticSearch`, it'll use `cls.get_es()`.
+        :arg es: The `Elasticsearch` to use. If you don't specify an
+            `Elasticsearch`, it'll use `cls.get_es()`.
 
         :arg index: The name of the index to use. If you don't specify one
             it'll use `cls.get_index()`.
@@ -2066,7 +2222,7 @@ class Indexable(object):
         if index is None:
             index = cls.get_index()
 
-        es.delete(index, cls.get_mapping_type_name(), id_)
+        es.delete(index=index, doc_type=cls.get_mapping_type_name(), id=id_)
 
     @classmethod
     def refresh_index(cls, es=None, index=None):
@@ -2078,8 +2234,8 @@ class Indexable(object):
         `refresh_index` as soon as you're done indexing. This is
         particularly helpful for unit tests.
 
-        :arg es: The `ElasticSearch` to use. If you don't specify an
-            `ElasticSearch`, it'll use `cls.get_es()`.
+        :arg es: The `Elasticsearch` to use. If you don't specify an
+            `Elasticsearch`, it'll use `cls.get_es()`.
 
         :arg index: The name of the index to use. If you don't specify one
             it'll use `cls.get_index()`.
@@ -2091,4 +2247,4 @@ class Indexable(object):
         if index is None:
             index = cls.get_index()
 
-        es.refresh(index)
+        es.indices.refresh(index=index)
